@@ -75,7 +75,7 @@ $registrar ||= $proxy;
 ###################################################
 my $leg;
 if ( !$proxy ) {
-	my ($host,$port) = $from =~m{\@([\w\-\.]+)(?:(\d+))?} 
+	my ($host,$port) = $from =~m{\@([\w\-\.]+)(?::(\d+))?} 
 		or die "cannot find SIP domain in '$from'";
 	my $addr = gethostbyname( $host )
 		|| die "cannot get IP from SIP domain '$host'";
